#!/usr/bin/perl
use strict;

my ($station, $tmpfolder, $proto) = @ARGV;
my ($time, $data);

#Create URI and get the data:
if ($proto eq "http") {
	my $URI = "http://weather.noaa.gov/cgi-bin/mgetmetar.pl?cccc=" . $station;
	my $tmpfile = "$tmpfolder/$station.TMP";
	`wget -q -O '$tmpfile' '$URI'`;
	if (-f $tmpfile && -r $tmpfile) {
		open (DATA, $tmpfile) || die "Couldn't open $tmpfile: $!";
		my $datastart = 0;
		while (<DATA>) {
			$datastart=1 if /START BODY OF TEXT HERE/;
			if ($datastart) {
				$time = $1 if m#^(\d{4}/\d\d?/\d\d? \d\d:\d\d)#;
				$data = $1 if m#^($station.+)$#;
			}
			last if $time && $data;
		}
		close DATA;
	}
	unlink $tmpfile;
} elsif ($proto eq "ftp") {
	my $URI = "ftp://weather.noaa.gov/data/observations/metar/stations/" . $station . ".TXT";
	my $tmpfile = "$tmpfolder/$station.TMP";
	`wget -q -O '$tmpfile' '$URI'`;
	if (-f $tmpfile && -r $tmpfile) {
		open (DATA, $tmpfile) || die "Couldn't open $tmpfile: $!";
		chomp($time = <DATA>);
		chomp($data = <DATA>);
		close DATA;
	}
	unlink $tmpfile;
}

exit -1 unless $time && $data;


my ($clouds, $weather, $hour, $minute, $winddir, $windspeed, $windgust, $temp, $dew) = ("","",0,0,-1,0,0);

#Time:
($hour, $minute) = split /:/, (split / /, $time)[1];

#Strip off METAR/SPECI
$data =~ s/^(METAR|SPECI) //;

#Strip off station name
$data =~ s/^$station //;

#Strip off time:
$data =~ s/^\S+ //;

#Strip off flag for automatic generation:
$data =~ s/^AUTO //;

#Wind:
my $wind = $1 if ($data =~ s/^(\S+) //);
if ($wind) {
	$winddir   = substr($wind,0,3,"");
	$windspeed = substr($wind,0,2,"");
	$windgust  = substr($wind,0,2,"") if ($wind =~ s/^G//);
	             substr($wind,0,length($wind),""); #Strip off unit
}

#Strip off visibility:
$data =~ s/^(\d+|\S+?SM) //;

#Weather and clouds:
while ($data !~ m#^\S+?/\S+?# && $data) {
	my $chunk = $1 if $data =~ s/^(\S+) //;

	#Describes clouds:
	if ($chunk eq "CAVOK") {
		$clouds .= "CAVOK,;";
	} elsif ($chunk =~ /^(CLR|SCK|FEW|SCT|BKN|OVC)(\d+)$/) {
		$clouds .= "$1,$2;";
		
	#Describes weather:
	} else {
		my ($intensity,$desc,$prec,$obsc,$other) = ("","","","","");
		$chunk =~ s/^VC//;
		$intensity = $1 if $chunk =~ s/^(\+|-)//;
		$desc      = $1 if $chunk =~ s/^(MI|PR|BC|DR|BL|SH|TS|FZ)//;
		$prec      = $1 if $chunk =~ s/^(DZ|RA|SN|SG|IC|PL|GT|GS|UP)//;
		$obsc      = $1 if $chunk =~ s/^(BR|FG|FU|VA|DU|SA|HZ|PY)//;
		$other     = $1 if $chunk =~ s/^(PO|SQ|FC|SS|SD)//;

		$weather  .= "$intensity,$desc,$prec,$obsc,$other;";
	}
}

($temp, $dew) = split m#/#, (split " ", $data)[0];
 $temp =~ s/^M/-/;
 $dew  =~ s/^M/-/;
 $weather ||= ",,,,;";
 $clouds  ||= ",,,,;"; 

exit -1 unless $temp && $dew;

open(FILE,"> $tmpfolder/$station") || die "Couldn't open $tmpfolder/$station:$!";
print FILE "Hour:$hour\n";
print FILE "Minute:$minute\n";
print FILE "Station:$station\n";
print FILE "WindDir:$winddir\n";
print FILE "WindSpeed:$windspeed\n";
print FILE "WindGust:$windgust\n";
print FILE "Weather:$weather\n";
print FILE "Clouds:$clouds\n";
print FILE "Temp:$temp\n";
print FILE "Dew:$dew\n";
close FILE;
